from aiohttp import web
if __name__=="__main__":
    from gbcall.settings import DEFAULT_PORT
    from gbcall import handler
else:
    from .settings import DEFAULT_PORT
    from . import handler

from colorama import Fore,Style

@web.middleware
async def responseJsonWrapper(request,handler):
    try:
        return web.json_response(
            {"res":await handler(request),"status":"OK"},
            headers={"Access-Control-Allow-Origin":"*"}
        )
    except Exception as e:
        if __name__!="__main__":
            raise e
        return web.json_response(
            {"res":str(e),"status":"ERR"},
            headers={"Access-Control-Allow-Origin":"*"}
        )

def run():
    app = web.Application(middlewares=[responseJsonWrapper])
    app.add_routes([
        web.get('/isAlive',handler.handshake),
        web.get('/funcRegister', handler.funcRegister),
        web.post('/call/{funcName}', handler.callFunc),
        web.get('/info/{funcName}', handler.getFuncInfo),
    ])
    print(f"python_server on: {Fore.RED}http://localhost:{DEFAULT_PORT}{Style.RESET_ALL}")
    web.run_app(app, port=DEFAULT_PORT, print=None)

if __name__=="__main__":
    run()


import os
from .utils import dynamicImport
from .settings import ALIVE_SYMBOL
from .typeConverter import convertParameters,typeNameConvert
from .scanner import scanner

infoCache = {}
moduleCache = {}
funcCache = {}

async def handshake(_request):
    return ALIVE_SYMBOL["python"]


async def funcRegister(request):
    file = request.rel_url.query['filePath']
    base = request.rel_url.query['workingPath']

    absPath = os.path.join(base, file)
    try:
        module = dynamicImport(absPath)
    except:
        raise Exception("no file or error in file")
    with open(os.path.join(base, file), 'r') as source:
        funcName, input, output = scanner(source.read())
        funcCache[funcName] = getattr(module,funcName)
        infoCache[funcName] = {
            "input": input,
            "output": output
        }
    return funcName


async def callFunc(request):
    data=await request.json()
    funcName = request.match_info.get('funcName', "Anonymous")
    res=funcCache[funcName](*convertParameters(infoCache[funcName]["input"],data["args"]))
    return convertParameters(infoCache[funcName]["output"],res)


def getFuncInfo(request):
    name = request.match_info.get('funcName')
    try:
        return infoCache[name]
    except:
        raise Exception("no such function")


import re
from .typeConverter import typeNameConvert

def scanner(code):
    regex = re.compile("@defineExpose\((.+)\)\ndef (.+):", re.S)
    IOStr = re.search(regex, code).group(1)
    SignStr = re.search(regex, code).group(2)
    args = re.search(re.compile(
        "input=\[(.*?)\]", re.S), IOStr).group(1).split(',')
    args = list(map(lambda s: s.strip(), args))
    args = list(filter(lambda s: s != '', args))
    ret = re.search(re.compile("output=\[(.*?)\]", re.S), IOStr).group(1).split(',')
    ret = list(map(lambda s: s.strip(), ret))
    ret = list(filter(lambda s: s != '', ret))
    funcName = re.search(re.compile("(.+?)\(.+?\)", re.S), SignStr).group(1)

    return funcName, list(map(typeNameConvert,args)), list(map(typeNameConvert,ret))



DESCRIPTION = "A simple serverless function transformer,\
        and a simple way to call function in other language."

DEFAULT_PORT = 23330
ALIVE_SYMBOL = {
    "python": "\"PYTHON_GLOBAL_CALL_ALIVE\""
}

OK_STATUS = "OK"



from setuptools import setup
from version import VERSION

setup(
    name='global-call-browser',  # package name
    version=VERSION,  # package version
    description='A simple serverless function transformer,\
     and a simple way to call function in other language.',  # package description
    packages=['gbcall'],
    # package_dir={"": "src"},
    zip_safe=False,
    author="bi1lqv",
    author_email="bi1lqy.y@gmail.com",
    license="MIT",
    install_requires=[],
    entry_points={}
)



from colorama import Fore, Style


import importlib.util
import sys

def mapAppend(m, key, val):
    if key in m:
        m.append(val)
    else:
        m[key] = [val]


def printError(txt):
    print(f"{Fore.RED}ERROR:{Style.RESET_ALL} {txt}")

def dynamicImport(path):
    spec = importlib.util.spec_from_file_location(
        path, path)
    foo = importlib.util.module_from_spec(spec)
    sys.modules[path] = foo
    spec.loader.exec_module(foo)
    return foo
